package com.bursie.harvester.gwt.client;

import com.google.gwt.core.client.EntryPoint;
import com.google.gwt.user.client.ui.*;
import com.google.gwt.user.client.Window;
import com.google.gwt.user.client.rpc.AsyncCallback;

import java.util.Arrays;
import java.util.List;

public class Harvester implements EntryPoint {
    private VerticalPanel mainPanel = new VerticalPanel();

    private HorizontalPanel searchPanel = new HorizontalPanel();
    private TextBox seedKeywordTextBox = new TextBox();
    private Button searchButton = new Button("Search");
    private Label informationLabel = new Label();

    private FlexTable suggestedKeywordsTable = new FlexTable();

    public void onModuleLoad() {

        // assemble Search panel
        searchPanel.add(seedKeywordTextBox);
        searchPanel.add(searchButton);

        // set up event listeners for searching using the button
        searchButton.addClickListener(new ClickListener() {
          public void onClick(Widget sender) {
            searchKeywords();
          }
        });

        // set up event listeners for searching using the keyboard
        seedKeywordTextBox.addKeyboardListener(new KeyboardListenerAdapter() {
          @Override
          public void onKeyDown(Widget sender, char keyCode, int modifiers) {
            if (keyCode == KEY_ENTER) {
              searchKeywords();
            }
          }
        });

        // assemble main panel
        mainPanel.add(searchPanel);
        mainPanel.add(informationLabel);
        mainPanel.add(suggestedKeywordsTable);

        // add the main panel to the HTML element with the id "harvester"
        RootPanel.get("harvester").add(mainPanel);

        // move cursor focus to the text box
        seedKeywordTextBox.setFocus(true);
    }

    private void searchKeywords() {
        final String seedKeyword = seedKeywordTextBox.getText().trim();
        seedKeywordTextBox.setFocus(true);

        //Telling the user that we're alive...
        informationLabel.setText("Searching...");

        AsyncCallback<List<Word>> callback = new AsyncCallback<List<Word>>() {
            public void onFailure(Throwable caught) {
                //this is not good error handling...
                caught.printStackTrace();
            }

            public void onSuccess(List<Word> result) {
                updateKeywordsTable(result);
            }
        };
        KeywordService.App.getInstance().getSuggestedKeywords(seedKeyword, callback);
    }

    private void updateKeywordsTable(List<Word> suggestedKeywords) {
        int rows = suggestedKeywordsTable.getRowCount();
        for (int i = 0; i< rows; i++) {
            suggestedKeywordsTable.removeRow(0);
        }
        informationLabel.setText("Found "+suggestedKeywords.size()+" related keywords");
        if (suggestedKeywords.isEmpty()) {
            seedKeywordTextBox.setFocus(true);
            seedKeywordTextBox.selectAll();
            return;
        }

        suggestedKeywordsTable.setText(0, 0, "Keyword");
        suggestedKeywordsTable.setText(0, 1, "Relevance");
        suggestedKeywordsTable.getRowFormatter().addStyleName(0, "resultHeader");

        int keywords = suggestedKeywords.size();
        for (int i = 1; i < suggestedKeywords.size()+1; i++) {
            suggestedKeywordsTable.setText(i, 0, suggestedKeywords.get(i-1).getKeyword());
            suggestedKeywordsTable.setText(i, 1, String.valueOf(suggestedKeywords.get(i-1).getRelevance()));
            if (isOdd(i)) {
                suggestedKeywordsTable.getRowFormatter().addStyleName(i, "resultOdd");
            } else {
                suggestedKeywordsTable.getRowFormatter().addStyleName(i, "resultEven");
            }
        }
    }

    private boolean isOdd(int rownum) {
        return rownum % 2 != 0;
    }
}
